{% extends "admin/change_form.html" %}

{% load admin_urls %}

{% block object-tools-items %}
  {{ block.super }}

  {% if original %}
    <li><a href="{% url opts|admin_urlname:'preview' original.pk %}">Preview</a></li>
  {% endif %}
{% endblock %}
