<p>Hello,</p>
<p>You have indicated that you wish to globally unsubscribe from all mailing lists at "{{ BASE_URL }}". If you have received this message in error, kindly ignore it.</p>
<p>If you do wish to globally unsubscribe, click the link below:</p>
<a href="{{ BASE_URL }}{% url 'mailinglist:subscribe_confirm' subscription.token %}">Confirm global unsubscribe</a>
<p>Thanks</p>
<p>- {{ DEFAULT_SENDER_NAME }}</p>
