<h1>Hello</h1>
<p>You are globally unsubscribing from "{{ BASE_URL }}".</p>
<p>You should be receiving an email shortly to confirm your email address. Click the link in that email to unsubscribe.
</p>
