# Generated by Django 4.2.7 on 2023-11-23 17:36

from django.db import migrations, models
import django.db.models.deletion
import mailinglist.models


class Migration(migrations.Migration):
    dependencies = [
        ("mailinglist", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="MessageAttachment",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "file",
                    models.FileField(
                        upload_to=mailinglist.models.attachment_upload_to,
                        validators=[mailinglist.models.hookset_validation_wrapper],
                        verbose_name="attachment",
                    ),
                ),
                ("filename", models.CharField(max_length=256)),
                (
                    "message",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attachments",
                        to="mailinglist.message",
                    ),
                ),
            ],
        ),
    ]
